      program piapp
      implicit none
      include 'mpif.h'

      double precision  delt, lo, hi, pia, pial, piexact, numint
      integer  i, ip, n, nla, nl
      integer comm, np, me, ierr
      piexact = 3.14159265358979323846264338327950288419d0 

! 1) set up mpi environment
      call MPI_INIT( ierr )
      comm = MPI_COMM_WORLD
      call MPI_COMM_SIZE( comm, np, ierr )
      call MPI_COMM_RANK( comm, me, ierr )
!  input of n
! 2) for me=0 : read the value for n from standard input
      if (me.eq.0) then
         write(6,*)'number of intervals ?  '
         read(5,*) n

         write(6,1001) n, np
 1001 format(//'calculation of pi using numerical integration of '
     :         'SQRT(1-x**2)'
     :        /'number of intervals    : ',I20,
     :        /'number of processes    : ',I20)
      end if

! 3) distribute n to all tasks
      if (me.eq.0) then
         do ip = 1 , np-1
            call MPI_SEND( n, 1, MPI_INTEGER, ip, 0, comm, ierr )
         end do
      else 
         call MPI_RECV( n, 1, MPI_INTEGER, 0, 0,
     :                  comm, MPI_STATUS_IGNORE,ierr )
      end if

! 4) determin local values for nl, lo, hi and calculate local integrals 
      pial = 0.d0
      nla = (n+np-1)/np
      delt = 1.d0/dble(n)
      nl = min(nla,n-me*nla)
      if (nl.gt.0) then
         lo = me*nla*delt
         pial = numint(lo,delt,nl,pial)
      end if

! 5) collect all partial sums on task 0
      if (me.eq.0) then
         pia = pial 
         do ip = 1 , np-1
            call MPI_RECV(pial, 1, MPI_DOUBLE_PRECISION, ip, ip,
     :                    comm, MPI_STATUS_IGNORE, ierr )
            pia = pia + pial
         end do
      else
         call MPI_SEND( pial, 1, MPI_DOUBLE_PRECISION, 0, me,
     :                  comm, ierr )
      end if

! 6) output of results
      if (me.eq.0) then
         write(6,1111) piexact, pia, abs(pia-piexact)
 1111 format(/'pi exact              = ',f20.14,
     :       /'pi estimated          = ',f20.14,
     :       /'error                 = ',e20.3)
      end if
      call MPI_FINALIZE(ierr)
      stop
      end
