      subroutine reduce_vector( n, y, x )
c     ===============================
c  all variables should be explicitely declared !
      implicit none 
c
      integer           n
      real*8            y(*), x(*)
c
c  +-------------+----------------+------------------------------------+
c  |  parameter  |  input/output  |  description                       |
c  +-------------+----------------+------------------------------------+
c  |  n          |  input         |  global vectorlength               |
c  |  y          |  input         |  partial sum of global vector      |
c  |  x          |  output        |  local elements of full vector     |
c  +-------------+----------------+------------------------------------+
c  |                                                                   |
c  |  Purpose of this subroutine is to add the partial result vectors  |
c  |  from all processes and scatter the result across the processes.  |
c  |                                                                   |
c  +-------------------------------------------------------------------+
c  |  ruler                                                            |
c  |5 7 10   15   20   25   30   35   40   45   50   55   60   65   70 | 
c  ++-+--+----+----+----+----+----+----+----+----+----+----+----+----+-+
c
c  parameter constants
      integer           nprocmax
      parameter         (nprocmax = 128)
c  FORTRAN MPI-INCLUDE-file
      include 'mpif.h'
c  local variables
      integer           ierr, nproc, myid, status(MPI_STATUS_SIZE)
      integer           firstind(0:nprocmax), counts(0:nprocmax)
     :                , ip, count, displ


      call MPI_COMM_SIZE( MPI_COMM_WORLD, nproc, ierr )
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )

      call dist_index( n, firstind )
      call MPI_REDUCE(y, x, n, MPI_DOUBLE_PRECISION,
     :                    MPI_SUM, 0, MPI_COMM_WORLD, ierr )
      if (myid.eq.0) then
         do ip = 1 , nproc-1
            displ = firstind(ip)+1
            count = firstind(ip+1) - firstind(ip)
            call MPI_SEND(x(displ),count,MPI_DOUBLE_PRECISION
     :                  , ip, 0, MPI_COMM_WORLD, ierr ) 
         end do
      else
         count = firstind(myid+1) - firstind(myid)
         call MPI_RECV(x,count,MPI_DOUBLE_PRECISION
     :               , 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE, ierr )
      end if

      return
      end
