      program ritz
c     ============
c
c  +-------------------------------------------------------------------+
c  |                                                                   |
c  |  This program determines the largest eigenvalue of a matrix using |
c  |  the method of Raleigh and Ritz.                                   |
c  |                                                                   |
c  |  Sequential version                                               |
c  |                                                                   |
c  +-------------------------------------------------------------------+
c  |  ruler                                                            |
c  |5 7 10   15   20   25   30   35   40   45   50   55   60   65   70 | 
c  ++-+--+----+----+----+----+----+----+----+----+----+----+----+----+-+
c
c  all variables should be explicitely declared !
      implicit none 
c  FORTRAN MPI-INCLUDE-file
      include 'mpif.h'
c  parameter constants
      integer           nmax 
      parameter         (nmax = 2**13)
c  local variables
      integer           i, j, n,  nloop, istop, ierr
      real*8            a(nmax * nmax),
     :                  x(nmax), y(nmax), 
     :                  flops, tcpu, tdelta, evn, evo, res
      real*8            t1, t2, t3, t4, t5, tmv, tcl, tte, tsc, tg
         call MPI_INIT( ierr )
         tg = MPI_WTIME()
c  get matrix size     
         print 6000
         read *, n
         if  (n .le. 0  .or.  n .gt. nmax)   then
            write(6,*)'n, nmax',n,nmax 
            stop 'n out of range'
         end if
         nloop = 1000
c  initialize global matrix
         print 6100
         do  j = 1, n
            do  i = 1 , j - 1
               a(i + n * (j - 1)) = a(j + n * (i - 1))
            end do
            do  i = j, n
               a(i + n * (j - 1)) = (dble(i) + dble(j))/dble(i)
            end do
         end do
c--------------------------------------------------------------
c  Raleigh-Ritz algoritm
c--------------------------------------------------------------
c  initialize start vector x
      do i = 1 , n
         x(i) = 1.0
      end do
      tmv = 0.0
      tcl = 0.0
      tte = 0.0
      tsc = 0.0
      tcpu = MPI_WTIME()
  
      evo = x(1)
      istop = 0
      do  i = 1, nloop
         call mv(n, n, a, x, y)
            evn = y(1)
            res = (evn - evo) / evn
            if (abs(res).le.1.0e-12) istop = 1
         write(6,*)i,evn,res
         if (istop.eq.1) go to 1
         do j = 1 , n
            x(j) = y(j) / evn
         end do
         evo = evn
      end do
 1    continue
      tcpu = MPI_WTIME() - tcpu
c  print results
         write(6,*)i,evn
         if (tcpu .gt. 0.0) then
            flops = 1e-6 * (2 * i * float(n) * float(n)) / tcpu
         else
            flops = 0.0
         endif
         print 6300, flops, tcpu
         tg = MPI_WTIME() - tg
         print 6400, tg
      call MPI_FINALIZE( ierr )
c  format statements
 6000 format(' ', /,
     :       ' Raleigh-Ritz running ...', //,
     :       ' N = ?')
 6100 format(' ', /,
     :       ' Initializing data ...')
 6200 format(' Raleigh-Ritz ...', /)
 6300 format(' MFlop/s ............................ : ', f12.4, /,
     :       ' CPU seconds ........................ : ', e12.4, /)
 6400 format( /,
     :       ' Total time ......................... : ', e12.4, /)
c  end of program ritz
      end




