      program demo_PDGEMV
!     ==================
!  +-------------------------------------------------------------------+
!  |  Demo program for using the PBLAS routine PDGEMV                  |
!  |  for calculating the matrix vector product with parameters        |
!  |  m_A = m, n_A = n, mb_A = mb, nb_A = nb, i_A = 1, j_A = 1         |
!  |  m_X = n, n_X = 1, mb_X = nb, nb_X = 1,  i_X = 1, j_X = 1         |
!  |  m_Y = m, n_Y = 1, mb_Y = mb, nb_Y = 1,  i_Y = 1, j_Y = 1         |
!  |  trans = 'N', incx = 1, incy = 1, csrc = 0, rsrc = 0              |
!  |  The matrices A and X are chosen as                               |
!  |  A(i,j) = mati*i + matj*j, X(i,1) = veci*i + vecj                 |
!  +-------------------------------------------------------------------+
!
!  all variables should be explicitely declared !
      implicit none 
!  FORTRAN MPI-INCLUDE-file
      include 'mpif.h'

!  parameter constants
      integer           max_al, max_xl, max_yl
      parameter        (max_al=10000, max_xl=10000,  max_yl=10000)
!  variables for definition of matrix A and matrix X
      real*8 mati,matj
      common/matdat/mati,matj
      real*8 veci,vecj
      common/matdat/veci,vecj
!  local variables
      real*8    al(max_al), xl(max_xl), yl(max_yl),
     :          matval, vecval, resval, res
      integer   ctxt, ctxt_sys, ctxt_all,
     :          nproc, nprow, npcol, myid, myrow, mycol,
     :          llda, lldx, lldy, desc_A(9), desc_X(9), desc_Y(9),
     :          m, n, mb, nb, m_al, n_al, m_xl, n_xl, m_yl, n_yl,
     :          ipr, ipc, il, jl, i, j, info, 
     :          NUMROC, INDXL2G, INDXG2L, INDXG2P 

!  calling BLACS_PINFO(myid,nproc) initializes MPI and returns size and rank
      call BLACS_PINFO(myid,nproc) 
      call BLACS_GET( 0, 0, ctxt_sys )
      ctxt_all = ctxt_sys
      call BLACS_GRIDINIT( ctxt_all, 'C', nproc, 1)
! input parameters
! Parameters for distributed global matrices A, X, Y
      m = 9; n = 7; mb=2; nb = 3
! Parameters for processor grid
      nprow=2; npcol=3
! Set up a process grid of size nprow*npcol
      if (nprow*npcol.gt.nproc) then
         write(6,*) 'nproc = ',nproc,' less then nprow*npcol = '
      end if
      ctxt = ctxt_sys; call BLACS_GRIDINIT( ctxt, 'C', nprow, npcol)
! Processes not belonging to the grid jump to the end of program
      if (ctxt.lt.0) go to 1000
! Get the process coordinates in the grid
      call BLACS_GRIDINFO( ctxt, nprow, npcol, myrow, mycol )
! number of rows and columns of local parts al, xl, yl for A, X, Y
      m_al = NUMROC( m, mb, myrow, 0, nprow )
      n_al = NUMROC( n, nb, mycol, 0, npcol )
      m_xl = NUMROC( n, nb, myrow, 0, nprow ); n_xl = 1
      m_yl = m_al; n_yl = 1
! Test for sufficient memory
      if (m_al*n_al.gt.max_al.or.
     :    m_xl*n_xl.gt.max_xl.or.
     :    m_yl*n_yl.gt.max_yl )then
       write(6,*)'not enough memory:  al ',m_al*n_al,max_al,
     :                               'xl', m_xl*n_xl,max_xl,
     :                               'yl', m_yl*n_yl,max_yl
       end if 
! set data for A and X; A(i,j) = mati*i + matj*j, X(i,1) = veci*i+vecj 
      mati = 1; matj = 1; veci=1; vecj = 1

! initializing descriptors for the distributed matrices A, X, Y:
      llda =  max(1,m_al); lldx =  max(1,m_xl); lldy = max(1,m_yl)
      call DESCINIT( desc_A, m, n, mb, nb, 0, 0, ctxt, llda, info )
      call DESCINIT( desc_X, n, 1, nb, 1, 0, 0, ctxt, lldx, info )
      call DESCINIT( desc_Y, m, 1, mb, 1, 0, 0, ctxt, lldy, info )

! initialize in parallel the local parts al of A
      do jl = 1 , n_al
         j = INDXL2G(jl,nb,mycol,0,npcol)
         do il = 1 , m_al
            i = INDXL2G(il,mb,myrow,0,nprow)
            al(il+m_al*(jl-1))=  mati*i+matj*j
         end do
      end do

! initialize in parallel the local parts xl of X
      do il = 1 , m_xl
         i = INDXL2G(il,nb,myrow,0,nprow)
         xl(il)= veci*i+vecj
      end do

! calculate matrix vector product
      call PDGEMV( 'N',m,n,1.d0,al,1,1,desc_A,
     :                          xl,1,1,desc_X,1,
     :                     0.d0,yl,1,1,desc_Y,1 )
! check the result 
      jl = INDXG2L(1,1,0,0,npcol)
      ipc = INDXG2P(1,1,0,0,npcol)
      do i = 1 , m
         il = INDXG2L(i,mb,0,0,nprow)
         ipr = INDXG2P(i,mb,0,0,nprow)

            if (ipr.eq.myrow.and.ipc.eq.mycol) then
               res =n*mati*i*vecj + n*(n+1)/2.d0*(matj*vecj+mati*i*veci)
     :       + n*(n+1)*(2*n+1)/6.d0 * matj*veci
             write(6,*)i,yl(il), res
               if(yl(il).ne.res) 
     :        write(6,*)i,yl(il), res
             end if
         end do

 1000 continue
      call BLACS_EXIT(0)

      stop
      end
