      program use_PDGEMR2D
!     ====================
!  +-------------------------------------------------------------------+
!  |  Example program for initializing a block-cyclic distribted matrix|
!  |  as used in PBLAS and Scalapack routines.                         |
!  |  In the exampel the block-cyclic distributed matrix needed as     |
!  |  is generated from an ordenary matrix                             |
!  |  on process 0 by the ScaLAPACK copy routine PDGEMR2D.             |
!  |                                                                   |
!  |  Paramters for the distributed matrix are                         |
!  |    dimension of matrix           m = 9,  n= 10                    |
!  |    blocksize for distribution    mb = 2, nb = 3                   |
!  |    grid dimensions               nprow = 3, npcol = 2             |
!  +-------------------------------------------------------------------+
      implicit none 
!  parameter constants
      integer       nmax
      parameter    (nmax = 1024)
!  local variables
      integer       ctxt, ctxt_0, ctxt_sys, ctxt_all,
     :              nproc, nprow, npcol, myid, myrow, mycol,
     :              desca(9), desca_0(9), NUMROC, ierr, info,
     :              m, n, mb, nb, ml, nl, ipr, ipc, i, j
      real*8        matval, a_0(nmax*nmax), al(nmax*nmax)

!  calling BLACS_PINFO(myid,nproc) initializes MPI and returns size and rank
      call BLACS_PINFO(myid,nproc) 

!  set parameters for the distribtued matrix
      m = 7;  n= 7           
      mb = 2; nb = 2     
      nprow = 3; npcol = 2 

! get the system default context
      call BLACS_GET( 0, 0, ctxt_sys )

! Set up a process grid ctxt_all of size nproc 
      ctxt_all = ctxt_sys
      call BLACS_GRIDINIT( ctxt_all, 'C', nproc, 1)

! Set up a process grid ctxt of size nprow*npcol
      ctxt = ctxt_sys
      call BLACS_GRIDINIT( ctxt, 'C', nprow, npcol)

! Set up a process grid  ctxt_0 of size 1 
      ctxt_0 = ctxt_sys
      call BLACS_GRIDINIT( ctxt_0, 'C', 1, 1)
      write(6,*)'ctxt',myrow,mycol,ctxt

! All processes not belonging to ctxt jump to the end of the program
      if (ctxt.lt.0) go to 1000

! Get the process coordinates in the grid with contet ctxt
      call BLACS_GRIDINFO( ctxt, nprow, npcol, myrow, mycol )

! Initializing the descriptor for the distributed mxn matrix a:
      ml =  max(1,NUMROC( m, mb, myrow, 0, nprow ))
      call DESCINIT( desca, m, n, mb, nb, 0, 0, ctxt, ml, info )
      write(6,*)'info descinit',myrow,mycol,info
! Set up a global matrix a_0 on a single process 
! and copy it as a block-cyclic distributed matrix 
! using PDGEMR2D

! Initialize the descriptor for the global mxn matrices a_0:
! The full descriptor has to be defined only on the process 0, on which
! the global matrix is defined.
! On all other processes desca_0(2)=-1 must be specified.
          desca_0(2) = -1; if (myid.eq.0) 
     :    call DESCINIT( desca_0, m, n, m, n, 0, 0, ctxt_0, m, info )

!  on proc 0 generate the global matrix a_0
      if (myid.eq.0) then
         do  j = 1, n
            do  i = 1 , m
               a_0(i + m * (j - 1)) = matval(i,j)
            end do
         end do
      end if

! distribute the global matrix a_0 from proc 0 
! to matrix a in the processes of grid ctxt
      call pdgemr2d(m, n, a_0, 1, 1, desca_0, al, 1, 1, desca, ctxt)

! Inspect the local elements of the distributed matrix on each process
      call BLACS_BARRIER(ctxt,'A')
      call sleep(1)
      if (m.le.20.and.n.le.20) then
      do ipc = 0 , npcol-1
         do ipr = 0, nprow-1
            if (ipr.eq.myrow.and.ipc.eq.mycol.and.n.le.10) then
                write(6,232) ipr, ipc
                ml = NUMROC( m, mb, myrow, 0, nprow )
                nl = NUMROC( n, nb, mycol, 0, npcol )
                do i = 1 , ml
                   write(6,233)(al(i+ml*(j-1)),j = 1,nl)
                end do
 232   format('local array al on process (',i2,','i2,')')
 233   format(10f8.0)           
            end if
            call BLACS_BARRIER(ctxt,'A')
            call sleep(1)
         end do
      end do
      end if
      call BLACS_BARRIER(ctxt,'A')
 1000  continue
      call BLACS_EXIT(0)

      stop
      end

      real*8 function matval(i,j)
      implicit none
      integer i,j
      matval = 100*i + j
      return 
      end 
