c===================================================================
c      Waermeleitungsgleichung
c      Message Passing (MPI-) Version 
c===================================================================
      program mpi_driver
      implicit none
      include 'mpif.h'
      integer      nmax, nmax2, nprocmax 
      parameter  ( nmax = 3200, nmax2 = nmax+2, nprocmax = 16)
c  local variables
      real*8       a(nmax2*nmax2), u(nmax2*nmax2)
      integer      firstind(0:nprocmax), firstind1(0:nprocmax), 
     :             firstind2(0:nprocmax),
     :             n1, n2, nq1, nq2, nt, i, nop, n1l, n2l, ip,
     :             com, ierr, nproc, myid, ist, istl, aus,
     :             ndim, dims(2), coords(2), com_2d, row
      logical      periods(2), reorder
      real*8       r, t1, t2, speed,
     :             c1, c2, c3, c4, c5, c6,
     :             trand, tupd, tav, tavl
      com = MPI_COMM_WORLD
      call MPI_INIT( ierr )
      call MPI_COMM_SIZE( com, nproc, ierr )
      call MPI_COMM_RANK( com, myid, ierr )

c------------ 1. Parametereingabe ---------------------------------
      aus = 2
      if (myid.eq.0) then
         read(5,*) nt
         read(5,*) n1, n2
         read(5,*) nq1, nq2
         read(5,*) r
         nq1 = sqrt(dble(nproc)); nq2 = nq1
         n1 = 50*nq1 ; n2 = n1
         write(6,*) n1,n2,nq1,nq2,nt,r
         if (r.gt.0.25) then
            write(6,*)
     :     'Warnung: r =',r,' ! Stabilitaet nur fuer r < 0.25 !' 
         end if
      end if
      call MPI_BARRIER( com, ierr )
      call MPI_BCAST( nt, 1, MPI_INTEGER, 0, com, ierr)
      call MPI_BCAST( n1, 1, MPI_INTEGER, 0, com, ierr)
      call MPI_BCAST( n2, 1, MPI_INTEGER, 0, com, ierr)
      call MPI_BCAST( nq1, 1, MPI_INTEGER, 0, com, ierr)
      call MPI_BCAST( nq2, 1, MPI_INTEGER, 0, com, ierr)
      call MPI_BCAST( r, 1, MPI_DOUBLE_PRECISION, 0, com, ierr)
      if (nproc.ne.nq1*nq2) then
         write(6,*)'nproc != nq1*nq2 : error'
         stop
      end if
      dims(1) = nq1; dims(2) = nq2;periods=.false.
      call MPI_CART_CREATE(com,2,dims,periods,.true.,com_2d,ierr)
      call MPI_CART_GET(com_2d,2,dims,periods,coords,ierr)
!      write(6,*)myid,coords(1),coords(2)

      call dist_index( n1, nq1, firstind1 )
      call dist_index( n2, nq2, firstind2 )
      n1l = firstind1(coords(1)+1) - firstind1(coords(1))
      n2l = firstind2(coords(2)+1) - firstind2(coords(2))
!      write(6,*) 'n1l, n2l ',myid, n1l, n2l
      istl = 0
      if (n1l*n2l.gt.nmax**2) then
         write(6,*)
     :'Speicherplatz fuer Gittergroesse nicht ausreichend:'
         write(6,*)
     :      'nmax =',nmax,' n1l =',n1l,' n2l =',n2l
         istl = -1
      end if
      call MPI_ALLREDUCE( istl, ist, 1, 
     :                    MPI_INTEGER, MPI_MIN, com, ierr )
      if (ist.lt.0) then
         call MPI_FINALIZE( ierr )
         stop
      end if
c------------ 2. Initialisierung der Anfangswerte u. Quellen -------
      call initialisierung_2d ( com_2d, n1l, n2l , a )
      call initialisierung_2d ( com_2d, n1l, n2l , u )
      call MPI_BARRIER( com, ierr )

c------------ 3. Zeitschritte zur Temperaturfeldberechnung ---------
      trand = 0.0
      tupd = 0.0
!      call MPI_TYPE_VECTOR(n2l, 1,  n1l+2, MPI_DOUBLE_PRECISION,
!     :                           row, ierr)
!      call MPI_TYPE_COMMIT(row,ierr)
      t1 = MPI_WTIME()
      do i = 1 , nt
         c1 = MPI_WTIME()
         call randaustausch( com_2d, n1l, n2l, a )
         c2 = MPI_WTIME()
         call zeitschritt ( r, n1l, n2l, a, u )
         c3 = MPI_WTIME()
         call randaustausch( com_2d, n1l, n2l, u )
         c4 = MPI_WTIME()
         call zeitschritt ( r, n1l, n2l, u, a )
         c5 = MPI_WTIME()
         trand = trand + c2 - c1 +c4-c3
         tupd = tupd + c3 - c2 +c5-c4
      end do
      t2 = MPI_WTIME() - t1
!      call MPI_Type_free(row,ierr)
      call temp(n1l,n2l,a,tavl)
      call MPI_REDUCE( tavl, tav, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0
     :               , com, ierr )

c------------ 4. Resultatausgabe - ---------------------------------
      if (MYID.eq.0) then
         if (nproc.eq.1) then
               call ausgabe_a ( n1, n2, a )
         end if
      write(6,'("-----------------------------")')
      write(6,'("  system size : ",i4," x ",i4)')n1,n2
      write(6,'("  after ",i4," timesteps :")') nt
      tav = tav /(real(n1*n2))
      write(6,'("  average temperature :", f10.5)') tav
      write(6,'("  run time : ", f10.4)') t2
      nop = 6*n1*n2
      speed = 1.e-6*nop*2*nt/t2
      write(6,'("  run speed [Mflop/s] :",f10.2)') speed

      write(6,*)'trand, tupd:'
      write(6,'(2f8.4)')trand, tupd
      end if

      call MPI_FINALIZE( ierr )
      stop
      end









