c===================================================================
c      Waermeleitungsgleichung
c      Message Passing (MPI-) Version 
c===================================================================
      program mpi_driver
      implicit none
      include 'mpif.h'
      integer      nmax, nmax2, nprocmax 
      parameter  ( nmax = 10000, nmax2 = nmax+2, nprocmax = 128)
c  local variables
      real*8       a(nmax2*nmax2), u(nmax2*nmax2)
      integer      firstind(0:nprocmax),
     :             n, nt, i, nop, nl, ip,
     :             com, ierr, nproc, myid, ist, istl
      real*8       r, t1, t2, speed, tav, tavl,
     :             c1, c2, c3, c4, c5,
     :             trand, trandl, tupd, tupdl

      com = MPI_COMM_WORLD
      call MPI_INIT( ierr )
      call MPI_COMM_SIZE( com, nproc, ierr )
      call MPI_COMM_RANK( com, myid, ierr )

c------------ 1. Parametereingabe ---------------------------------
      if (myid.eq.0) then
         read(5,*) nt
         read(5,*) n
         read(5,*) r
         n = 50.0*sqrt(real(nproc))
         write(6,*) n,nt,r
         if (r.gt.0.25) then
            write(6,*)
     :     'Warnung: r =',r,' ! Stabilitaet nur fuer r < 0.25 !' 
         end if
      end if
      call MPI_BARRIER( com, ierr )
      call MPI_BCAST( nt, 1, MPI_INTEGER, 0, com, ierr)
      call MPI_BCAST( n, 1, MPI_INTEGER, 0, com, ierr)
      call MPI_BCAST( r, 1, MPI_DOUBLE_PRECISION, 0, com, ierr)

      call dist_index( n, firstind )
      nl = firstind(myid+1) - firstind(myid)

      istl = 0
      if (n*nl.gt.nmax**2) then
         write(6,*)
     :'Speicherplatz fuer Gittergroesse nicht ausreichend:'
         write(6,*)
     :      'nmax =',nmax,' n =',n,'nl =',nl
         istl = -1
      end if
      call MPI_ALLREDUCE( istl, ist, 1, 
     :                    MPI_INTEGER, MPI_MIN, com, ierr )
      if (ist.lt.0) then
         call MPI_FINALIZE( ierr )
         stop
      end if
c------------ 2. Initialisierung der Anfangswerte u. Quellen -------
      call initialisierung ( n, nl , a )
      call initialisierung ( n, nl , u )
      call MPI_BARRIER( com, ierr )
c------------ 3. Zeitschritte zur Temperaturfeldberechnung ---------
      trandl = 0.0
      tupdl = 0.0
      t1 = MPI_WTIME()
      do i = 1,nt
         c1 = MPI_WTIME()
         call randaustausch( n, nl, a )
         c2 = MPI_WTIME()
         call zeitschritt ( r, n, nl, a, u )
         c3 = MPI_WTIME()
         call randaustausch( n, nl, u )
         c4 = MPI_WTIME()
         call zeitschritt ( r, n, nl, u, a )
         c5 = MPI_WTIME()
         trandl = trandl + c2-c1 + c4-c3
         tupdl = tupdl + c3-c2 + c5-c4
      end do
 4    continue
      call MPI_REDUCE( trandl, trand, 1, MPI_DOUBLE_PRECISION
     :              , MPI_MAX, 0, com, ierr )
      call MPI_REDUCE( tupdl, tupd, 1, MPI_DOUBLE_PRECISION
     :              , MPI_MAX, 0, com, ierr )
      call temp(n,nl,a,tavl)
      call MPI_REDUCE( tavl, tav, 1, MPI_DOUBLE_PRECISION
     :              , MPI_SUM, 0, com, ierr )
c------------ 4. Resultatausgabe - ---------------------------------
      call MPI_BARRIER( com, ierr )
      if (MYID.eq.0) then
      t2 = MPI_WTIME() - t1
      if (nproc.eq.1) then
               call ausgabe_a ( n, n, a )
      end if
      write(6,'("-----------------------------")')
      write(6,*)'number of processes : ',nproc
      write(6,'("  system size : ",i4," x ",i4)')n,n
      write(6,'("  after ",i4," timesteps :")') nt
      tav = tav /(real(n)**2)
      write(6,'("  average temperature :", f10.5)') tav
      write(6,'("  run time : ", f10.4)') t2
      nop = 6*n*n
      speed = 1.e-6*nop*2*nt/t2
      write(6,'("  run speed [Mflop/s] :",f10.2)') speed

      write(6,*)'trand, tupd:'
      write(6,'(2f8.4)')trand, tupd
      end if


      call MPI_FINALIZE( ierr )
      stop
      end








